def help():
    print("Welcome to Quito!")
    print("\n")
    print("You can provide the root of the configuration to run quito.")
    print("Here is the template of the configuration file.")

    template = '''
[program]
root= 
;(Required)
;Description: The absolute root of your quantum program file.
num_qubit= 
;(Required)
;Description: The total number of qubit of your quantum program.
inputID= 
;(Required)
;Description: The ID of input qubits.
outputID= 
;(Required)
;Description: The ID of output qubits which are the qubits to be measured.
;Format: A non-repeating sequence separated by commas.

[quito_configuration]
coverage_criterion= 
;Description: The coverage criterion you choose.
;Choice: IC/OC/IOC
K= 
;(Optional)
;Description: The total number of test suites, K=200 by default.
M= 
;(Optional)
;Description: The number of test suite groups, M=20 by default.
BUDGET= 
;(Optional)
;Description: The budget of the number of test cases in one test suite, BUDGET=10*number of inputs by default.
confidence_level= 
;(Optional)
;Description: The confidence level for statistical test, confidence_level=0.01 by default.
statistical_test= 
;(Optional)
;Description: The statistical test for assessment, statistical_test=one-sample Wilcoxon signed rank test by default.

[program_specification_category]
ps_category= 
;(Required) Description: The category of your program specification. (full/partial/no)

[program_specification]
;(Required for full and partial program specification)
;Description: The program specification.
;Format:input string,output string=probability
'''
    print(template)
    print('\n')

    print("Here is the example of the configuration file and the quantum program file.")

    example = '''
[program]
root=SWAP.py
num_qubit=3
inputID=0,1
outputID=2

[program_specification_category]
ps_category=full

[quito_configuration]
coverage_criterion=IC
K=200
M=10
BUDGET=20
confidence_level=0.01
statistical_test=one-sample Wilcoxon signed rank test    

[program_specification]
01,1=0.5
01,0=0.5
00,1=1
11,1=1
10,1=0.5
10,0=0.5
    '''
    program = '''
def run(qc):
    qc.h(2)
    qc.cswap(2,0,1)
    qc.h(2)
    qc.x(2)

    qc.measure(2,0)
    '''
    print(example)
    print("Here is the example of the quantum program file.")
    print(program)

if __name__ == '__main__':
    help()


from qusbt_tool.search_method import Search
import time
import os
import configparser
import sys
import warnings

curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.append(os.path.split(rootPath)[0])

if sys.platform.startswith('win32'):
    ROOT='\\'
elif sys.platform.startswith('linux'):
    ROOT='/'
elif sys.platform.startswith('darwin'):
    ROOT='/'

def qusbt(
        con_file: str
):
    warnings.filterwarnings('ignore')
    print("qusbt")
    _qusbt_run(con_file)

def _dec2bin(value, n):
    a = 1
    list = []
    while a > 0:
        a, b = divmod(value, 2)
        list.append(str(b))
        value = a
    s = ""
    for i in range(len(list) - 1, -1, -1):
        s += str(list[i])
    s = s.zfill(n)
    return s

def _expand_ps(ps,inputID,outputID):#check and expand
    flag = True
    for i in range(len(ps)):
        pair = ps[i][0].split(',')
        if len(pair[0]) != len(inputID) or len(pair[1]) != len(outputID):

            flag = False
            break
        for j in range(len(inputID)):
            if ps[i][0][j] != '0' and ps[i][0][j] != '1' and ps[i][0][j] != '-':
                flag = False
        for j in range(len(inputID)+1, len(inputID) + len(outputID) + 1):
            if ps[i][0][j] != '0' and ps[i][0][j] != '1' and ps[i][0][j] != '-':
                flag = False
    if flag == False:
        print("Error: The format of the program specification is wrong.")
        print("Example: 00-1,01=0.5")
        _end_running()
    else:
        l = len(inputID) + len(outputID) + 1
        length_origin = len(ps)
        items = []
        ps_new = []

        for i in range(length_origin):
            if ps[i][0].find('-',0,len(ps[i][0])) != -1:
                mark = []
                item = ps[i]
                items.append(item)
                for j in range(l):
                    if ps[i][0][j] == '-':
                        mark.append(j)
                for j in range(pow(2,len(mark))):
                    t=_dec2bin(j,len(mark))
                    temp = list(item[0])
                    for k in range(len(mark)):
                        temp[mark[k]] = t[k]
                    temp = ''.join(temp)
                    temp_item = (temp,item[1])
                    ps_new.append(temp_item)
        if len(items) == 0:
            return ps
        else:
            for i in range(len(items)):
                ps.remove(items[i])
            ps += ps_new
            return ps

def _check_configuration_file(config):
    if config.has_section('program') == False:
        print("Error: QuSBT cannot find section 'program' in this configuration file.")
        _end_running()
    else:
        if config.has_option('program', 'root') == False:
            print("Error: QuSBT cannot find the root of the quantum program file.")
            _end_running()
        if config.has_option('program', 'num_qubit') == False:
            print("Error: QuSBT cannot find the number of qubits of the program.")
            _end_running()
        if config.has_option('program', 'inputID') == False:
            print("Error: QuSBT cannot find the input IDs of the program.")
            _end_running()
        if config.has_option('program', 'outputID') == False:
            print("Error: QuSBT cannot find the output IDs of the program.")
            _end_running()

    if config.has_section('program_specification') == False:
        print("Error: QuSBT cannot find section 'program_spacification' in this configuration file.")
        _end_running()



def _check_unique(l):
    return len(l) == len(set(l))

def _check_unique_pair(ps):
    pair = []
    for i in range(len(ps)):
        pair.append(ps[i][0])
    return len(pair) == len(set(pair))

def _check_inputID_outputID(num_qubit, inputID, outputID):
    if _check_unique(inputID) == False:
        print("Wrong format of 'inputID'.")
        _end_running()
    if _check_unique(outputID) == False:
        print("Wrong format of 'outputID'.")
        _end_running()
    try:
        inputID = [int(i) for i in inputID]
    except:
        print("Wrong format of 'inputID'.")
        _end_running()
    try:
        outputID = [int(i) for i in outputID]
    except:
        print("Wrong format of 'outputID'.")
        _end_running()
    inputID.sort()
    outputID.sort()

    if int(inputID[-1]) > num_qubit - 1:#the last one
        print("Wrong input IDs")
        _end_running()
    if int(inputID[-1]) > num_qubit - 1:
        print("Wrong output IDs")
        _end_running()

    return inputID, outputID

def _check_bin(bin_str, n):
    if len(bin_str) != n:
        print("Error: The format of the program specification is wrong.")
        _end_running()
   # print("check bin: "+str(bin_str))
    for i in range(len(bin_str)):
        if bin_str[i] != '0' and bin_str[i] != '1':
            print("Error: The format of the program specification is wrong.")
            _end_running()

def _input_group(valid_input):
    index = [] #unique input index
    index_flag = valid_input[0]
    index.append(0)
    for i in range(1,len(valid_input)):
        if valid_input[i] != index_flag:
            index.append(i)
            index_flag = valid_input[i]
    return index

def _check_full_ps(valid_input, p):
    index = _input_group(valid_input)
    p_sum = 0
    for i in range(len(index)):
        start = index[i]
        if i == len(index) - 1:
            end = len(valid_input)
        else:
            end = index[i+1]
        for j in range(start,end):
            p_sum += p[j]
        if p_sum != 1:
            print("Error: This is not a complete program specification.")
            _end_running()
        else:
            p_sum = 0

def _end_running():
    exit()



def _qusbt_run(root_con):
    START = time.time()

    if os.path.isfile(root_con) == True:
        config = configparser.ConfigParser(allow_no_value=True)
        config.read(root_con, encoding='utf-8')
    else:
        print("Error: QuSBT cannot find the configuration file.")
        _end_running()

    _check_configuration_file(config)

    # get quantum program
    root = config.get('program', 'root')
    if os.path.isfile(root) != True:
        print("Error: QuSBT cannot find the quantum program file.")
        _end_running()

    root_list = root.split(ROOT)
    program_file = root_list[len(root_list) - 1]
    program_folder = root_list[:len(root_list) - 1]
    program_folder = ROOT.join(str(i) for i in program_folder)
    sys.path.append(program_folder)
    # print(program_file.split('.')[0])
    module_name = program_file.split('.')[0]

    # get inputID, outputID and numner of qubits
    try:
        num_qubit = int(config.get('program', 'num_qubit'))
    except:
        print("Error: The data type of 'num_qubit' should be an Integer")
        _end_running()
    inputID_o = config.get('program', 'inputID').split(',')
    outputID_o = config.get('program', 'outputID').split(',')
    inputID, outputID = _check_inputID_outputID(num_qubit, inputID_o, outputID_o)

    population_size = 10
    offspring_population_size = 10
    max_evaluations = 500
    if config.get('GA_parameter', 'population_size') != None:
        try:
            population_size = int(config.get('GA_parameter', 'population_size'))
        except:
            print("Error: The data type of 'population_size' should be an Integer")
            _end_running()
    if config.get('GA_parameter', 'offspring_population_size') != None:
        try:
            offspring_population_size = int(config.get('GA_parameter', 'offspring_population_size'))
        except:
            print("Error: The data type of 'off_population_size' should be an Integer")
            _end_running()

    if config.get('GA_parameter', 'max_evaluations') != None:
        try:
            max_evaluations = int(config.get('GA_parameter', 'max_evaluations'))
        except:
            print("Error: The data type of 'max_evaluations' should be an Integer")
            _end_running()

    if os.path.isfile(root) != True:
        print("Error: QuSBT cannot find the quantum program file.")
        _end_running()

    M_flag = False
    beta = 0
    if config.has_option('qusbt_configuration','M') == True:
        M_flag == True
        try:
            M = int(config.get('qusbt_configuration','M'))
        except:
            print("Error: The data type of 'M' should be an Integer.")
            _end_running()
        if config.has_option('qusbt_configuration', 'beta') == True:
            print("Error: You should use either 'M' or 'beta'.")
            _end_running()
    elif config.has_option('qusbt_configuration','beta') == True:
        try:
            beta = float(config.get('qusbt_configuration','beta'))
        except:
            print("Error: The data type of 'beta' should be a Float.")
            _end_running()
    else:
        beta = 0.05

    confidence_level = 0.01
    if config.has_option('qusbt_configuration', 'confidence_level') == True:
        try:
            confidence_level = float(config.get('qusbt_configuration', 'confidence_level'))
        except:
            print("Error: The data type of 'confidence_level' should be a Float.")
            _end_running()

    mutation_probability_flag = False
    if config.has_option('GA_configuration', 'mutation_probability') == True:
        mutation_probability_flag = True
        try:
            mutation_probability = float(config.get('GA_configuration', 'crossover_probability'))
        except:
            print("Error: The data type of 'mutation_probability' should be a Float.")
            _end_running()
        if mutation_probability >= 1:
            print("Error: The value of 'mutation_probability' should be a Float smaller than 1.0.")
            _end_running()

    mutation_distribution_index = 20
    if config.has_option('GA_configuration', 'mutation_distribution_index') == True:
        try:
            mutation_distribution_index = int(config.get('GA_configuration', 'mutation_distribution_index'))
        except:
            print("Error: The data type of 'mutation_distribution_index' should be an Integer.")
            _end_running()

    crossover_distribution_index = 20
    if config.has_option('GA_configuration', 'crossover_distribution_index') == True:
        try:
            crossover_distribution_index = int(config.get('GA_configuration', 'crossover_distribution_index'))
        except:
            print("Error: The data type of 'crossover_distribution_index' should be an Integer.")
            _end_running()

    crossover_probability = 0.9
    if config.has_option('GA_configuration', 'crossover_probability') == True:
        try:
            crossover_probability = float(config.get('GA_configuration', 'crossover_probability'))
        except:
            print("Error: The data type of 'crossover_probability' should be a Float.")
            _end_running()
        if crossover_probability >= 1:
            print("Error: The value of 'crossover_probability' should be a Float smaller than 1.0.")
            _end_running()


    #PS
    valid_input = []
    valid_output = []
    p = []
    ps = config.items('program_specification')
    ps = _expand_ps(ps, inputID, outputID)

    # print(ps)
    #print("origin:"+str(ps))
    if _check_unique_pair(ps) == False:
        print("There are duplicate input-output pairs in the program specification.")
        _end_running()
    #sort PS according to input and output
    ps.sort(key=lambda x:x[0])
    #print("new:"+str(ps))
    for i in range(len(ps)):
        valid_input_item = ps[i][0][:len(inputID)]
        valid_output_item = ps[i][0][len(inputID)+1:]
        _check_bin(valid_input_item,len(inputID))
        _check_bin(valid_output_item,len(outputID))
        valid_input.append(valid_input_item)
        valid_output.append(valid_output_item)
        p.append(float(ps[i][1]))
    _check_full_ps(valid_input, p)

    config_dic = {}
    config_dic['program_folder'] = program_folder
    config_dic['root'] = root_con
    config_dic['module_name'] = module_name
    config_dic['num_qubit'] = num_qubit
    config_dic['inputID'] = inputID
    config_dic['outputID'] = outputID
    if M_flag == True:
        config_dic['M'] = M
    else:
        config_dic['beta'] = beta
    config_dic['confidence_level'] = confidence_level
    config_dic['population_size'] = population_size
    config_dic['offspring_population_size'] = offspring_population_size
    config_dic['max_evaluations'] = max_evaluations
    config_dic['valid_input'] = valid_input
    config_dic['valid_output'] = valid_output
    config_dic['p'] = p
    config_dic['crossover_probability'] = crossover_probability
    config_dic['crossover_distribution_index'] = crossover_distribution_index
    if mutation_probability_flag == True:
        config_dic['mutation_probability'] = mutation_probability
    config_dic['mutation_distribution_index'] = mutation_distribution_index
    Search.search(config_dic)
    # Search.search(root, module_name, alg, len(inputID), beta)
    END = time.time()
    T1 = END-START



from qiskit import (
    #IBMQ,
    QuantumCircuit,
    QuantumRegister,
    ClassicalRegister,
    execute,
    Aer,
)
import importlib
import numpy as np
import time
import scipy.stats as stats


def dec2bin(value, n):
    a = 1
    list = []
    while a > 0:
        a, b = divmod(value, 2)
        list.append(str(b))
        value = a
    s = ""
    for i in range(len(list) - 1, -1, -1):
        s += str(list[i])
    s = s.zfill(n)
    return s


def wrong_output(i, right_output):
    set_output = set(right_output)
    if i not in set_output:
        return True  # existing wrong output
    return False

def execute_quantum_program(inputID, outputID, num_qubit, i, module_name, times): #module_name is the name of the quantum program file
    q = QuantumRegister(num_qubit)
    c = ClassicalRegister(len(outputID))
    qc = QuantumCircuit(q, c)
    i = dec2bin(i, len(inputID)) # Turning one integer into binary number
    for j in range(len(inputID)):
        if i[len(inputID) - 1 - j] == '1':
            qc.x(inputID[j])
    module = importlib.import_module(module_name)
    run_method = getattr(module,"run")
    run_method(qc)
    simulator_gpu = Aer.get_backend('aer_simulator')
    simulator_gpu.setoptions(device='GPU')
    result = execute(qc, simulator_gpu, shots=times).result().get_counts(qc)
    return result

def check_WOO(i, o, valid_inputs, valid_outputs):
    flag = False
    for k in range(len(valid_inputs)):
        if valid_inputs[k] == i and valid_outputs[k] == o:
            flag = True
    if flag == False:
        return False
    return True


def process_bar(percent, start_str='', end_str='', total_length=0):
    bar = ''.join(['#'] * int(percent * total_length)) + ''
    bar = '\r' + start_str + bar.ljust(total_length) + ' {:0>4.1f}%|'.format(percent*100) + end_str
    print(bar, end='', flush=True)

def calculate_fail_number_GA(input, config_dic, solution_sheet, log_sheet):
    count_fail = 0  # the number that fails the test
    pvalue = []
    global f
    valid_input = config_dic['valid_input']
    valid_output = config_dic['valid_output']
    pt = config_dic['p']
    inputs_str = []
    test_results = []


    for i in input:  # fix
        flag_wrong = False  # have wrong outputs
        fre = []  # counts of expected outputs
        p = []  # possibility of expected outputs
        outputs = []

        count_times = 0
        right_output = []
        i_str = dec2bin(i, len(config_dic['inputID']))
        inputs_str.append(i_str)
        for j in range(len(valid_input)):
            if valid_input[j] == i_str and pt[j] > 0: # finding valid outputs and probability of the input
                right_output.append(valid_output[j])
                p.append(pt[j])
                outputs.append(valid_output[j])
                count_times += 1

        # calculate outputs

        ##
        start = time.time()
        temp = execute_quantum_program(config_dic['inputID'], config_dic['outputID'], config_dic['num_qubit'], i, config_dic['module_name'], count_times*100)
        end = time.time()

        exe_time = solution_sheet['B4'].value + end - start

        solution_sheet['B4'] = exe_time

        # judge wrong outputs
        for key in list(temp.keys()):
            if key not in right_output:
                flag_wrong = True


        # chi test
        if flag_wrong == False:  # no wrong output
            if count_times == 1:  # only one output
                pvalue.append(1)
            else:
                for j_s in outputs:
                    if j_s in temp:
                        fre.append(temp[j_s])
                    else:
                        fre.append(0)
                p = np.array(p)
                fre = np.array(fre)
                fre = [x/(sum(fre)) for x in fre]
                # p = robjects.FloatVector(p)
                # fre = robjects.FloatVector(fre)

                # robjects.r('''
                #     chitest<-function(observed,theoretical){
                #         test_result <- chisq.test(x = observed,p = theoretical)
                #         pvalue = test_result$p.value
                #         return (pvalue)
                #     }''')
                # t = robjects.r['chitest'](fre, p)
                # pvalue.append(t[0])

                result = stats.chisquare(f_obs=fre, f_exp=p)
                pvalue.append(result[1])
        else:
            pvalue.append(0)
        if pvalue[-1] < 0.01:
            count_fail += 1
            if flag_wrong == False:
                test_result = 'wodf'
            else:
                test_result = 'uof'
        else:
            test_result = 'pass'
        test_results.append(test_result)


    count_fail_list = [count_fail]
    count_fail = np.array(count_fail_list)


    inputs_str.insert(0,'value of inputs')
    test_results.insert(0,'test results')
    count_fail_list.insert(0,'number of failing inputs')
    log_sheet.append(inputs_str)
    log_sheet.append(test_results)
    log_sheet.append(count_fail_list)
    process_bar((log_sheet.max_row / 3) / config_dic['max_evaluations'], start_str='', end_str='100%', total_length=30)

    return count_fail




import numpy as np
from jmetal.core.problem import IntegerProblem
from jmetal.core.solution import IntegerSolution

from qusbt_tool.testing import calculate_fail_number_GA

import math


class TestingProblem(IntegerProblem):
    def __init__(self, config_dic, solution_sheet, log_sheet):
        super(IntegerProblem).__init__()

        n = len(config_dic['inputID'])
        if 'M' in config_dic.keys():
            self.number_of_variables = config_dic['M']
        else:
            self.number_of_variables = math.ceil(pow(2, n) * config_dic['beta'])
        self.number_of_objectives = 1
        self.number_of_constraints = 0


        self.obj_directions = [self.MINIMIZE]

        self.lower_bound = self.number_of_variables * [0]

        self.upper_bound = self.number_of_variables * [pow(2,n)-1]

        self.config_dic = config_dic
        self.solution_sheet = solution_sheet
        self.log_sheet = log_sheet


    def evaluate(self, solution: IntegerSolution) -> IntegerSolution:
        variables = np.array(solution.variables)
        p = calculate_fail_number_GA(variables, self.config_dic, self.solution_sheet, self.log_sheet)
        # the value is negated because we want to maximize "p" using a minimization problem
        solution.objectives[0] = -p
        return solution

    def get_name(self) -> str:
        return 'TestingProblem'



def help():
    print("Welcome to QuSBT!")
    print("\n")
    print("You can provide the root of the configuration to run qusbt.")
    print("Here is the template of the configuration file.")

    template = '''
[program]
root= 
;(Required)
;Description: The absolute root of your quantum program file.
num_qubit= 
;(Required)
;Description: The total number of qubit of your quantum program.
inputID= 
;(Required)
;Description: The IDs of input qubits.
;Format: A non-repeating sequence separated by commas.
outputID= 
;(Required)
;Description: The IDs of output qubits which are the qubits to be measured.
;Format: A non-repeating sequence separated by commas.

[qusbt_configuration]
beta=
;(Optional)
;Description: The percentage of possible inputs as the number of test cases in a test suite.
M=
;(Optional)
;Description: The number of test cases in a test suite.
;Attention: You should use either 'beta' or 'M'. We use 'beta' as 0.05 by default.

[GA_parameter]
population_size= 
;(Optional) 
;Description: The population size in GA, population_size=10 by default.
offspring_population_size= 
;(Optional)
;Description: The offspring population size in GA, offspring_population_size=10 by default.
max_evaluations=
;(Optional)
;Description: The maximum evaluations in GA, max_evaluations=500 by default.
mutation_probability=
;(Optional)
;Description: mutation probability in GA, mutation_probability=1.0/M, 'M' is the size of a test suite by default.
mutation_distribution_index=
;(Optional)
;Description: mutation distribution in GA, mutation_distribution_index=20 by default.
crossover_probability=
;(Optional)
;Description: crossover probability in GA, crossover_probability=0.9 by default.
crossover_distribution_index=
;(Optional)
;Description: crossover probability in GA, crossover_distribution_index=20 by default.

[program_specification]
;Description: The program specification.
;Format:input string (binary),output string (binary)=probability
;Example:
;00,1=0.5
;00,0=0.5
;01,1=0.5
;01,0=0.5
;or
;0-,-=0.5
;Attention: '-' can refer to both '0' and '1'.
'''
    print(template)
    print('\n')

    print("Here is the example of the configuration file and the quantum program file.")

    example = '''
[program]
root=C:\IQ.py
num_qubit=10
inputID=0,1,2,3,4,5,6,7,8,9
outputID=0,1,2,3,4,5,6,7,8,9

[qusbt_configuration]
beta=0.05
confidence_level=0.01

[GA_parameter]
population_size=10
offspring_population_size=10
max_evaluations=500  

[program_specification]
----------,----------=0.0009765625
    '''
    program = '''
import math

def swap_registers(circuit, n):
    for qubit in range(n//2):
        circuit.swap(qubit, n-qubit-1)
    return circuit

def qft_rotations(circuit, qubit, p):
    for j in range(qubit + 1, 10):
        circuit.cu1(math.pi/2**(p), qubit, j)
        p += 1
    circuit.barrier()
    return circuit

def run(qc):
    n = 10
    swap_registers(qc,n)
    for qubit in range(n):
        qc.h(qubit)
        p = 1
        qft_rotations(qc,qubit,p)

    qc.measure([0,1,2,3,4,5,6,7,8,9],[0,1,2,3,4,5,6,7,8,9])
    '''
    print(example)
    print("Here is the example of the quantum program file.")
    print(program)

if __name__ == '__main__':
    help()






import math

def run(qc):
    qc.h(2)
    qc.p(math.pi/4, 2)

    qc.x(10)
    qc.h(11)
    qc.p(math.pi/2,11)

    for i in range(9):
        control = []
        control.append(10)
        for j in range(9-i):
            control.append(j)
        qc.mct(control, 9-i)
    qc.cnot(10,0)

    qc.barrier()

    for i in range(8):
        control = []
        control.append(10)
        control.append(11)
        for j in range(1,9-i):
            control.append(j)
        qc.mct(control,9-i)
    qc.ccx(10,11,1)

    qc.barrier()

    for i in range(8):
        control = []
        control.append(10)
        control.append(11)
        for j in range(1,9-i):
            control.append(j)
        qc.mct(control,9-i)
    qc.ccx(10, 11, 1)

    qc.barrier()

    for i in range(7):
        control = []
        control.append(11)
        for j in range(2,9-i):
            control.append(j)
        qc.mct(control,9-i)
    qc.cnot(11,2)

    qc.barrier()

    qc.measure([0,1,2,3,4,5,6,7,8,9], [0,1,2,3,4,5,6,7,8,9])


import setuptools

with open("README.md", "r") as f:
    long_description = f.read()

setuptools.setup(
    name="qusbt-gpu",
    version="1.0.0",
    author="Xinyi Wang",
    author_email="xinyi@simula.no",
    description="Search-based testing of quantum programs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    url="https://github.com/Simula-COMPLEX/qusbt-tool",
    classifiers= ["Programming Language :: Python :: 3",  # 编程语言
        "License :: OSI Approved :: MIT License",  # license
        "Operating System :: OS Independent"],  # 操作系统
    install_requires=[
        "qiskit",
        "qiskit-aer-gpu",
        "jmetalpy",
        "openpyxl"
    ],
    package_data={"pipmodule": ["*.png", ]},
    python_requires=">=3.7",
    entry_points={
        'console_scripts': [
            'qusbt = qusbt:help'
 ]
    }
)


from openpyxl import Workbook
from qusbt_tool.testing import dec2bin

import numpy as np
from jmetal.algorithm.singleobjective import GeneticAlgorithm
from jmetal.operator import IntegerPolynomialMutation
from jmetal.operator.crossover import IntegerSBXCrossover
from jmetal.operator.selection import BestSolutionSelection
from jmetal.util.termination_criterion import StoppingByEvaluations

from qusbt_tool.TestingProblem import TestingProblem

import sys
if sys.platform.startswith('win32'):
    ROOT='\\'
elif sys.platform.startswith('linux'):
    ROOT='/'
elif sys.platform.startswith('darwin'):
    ROOT='/'

class Search:


    @staticmethod
    def search(config_dic):
        wb = Workbook()
        solution_sheet = wb.active
        solution_sheet.title = 'Best Solution'
        log_sheet = wb.create_sheet('Log Information')
        solution_sheet['A4'] = 'Simulation Time'
        solution_sheet['B4'].value = 0

        problem = TestingProblem(config_dic, solution_sheet, log_sheet)
        np.set_printoptions(linewidth=500)

        if 'mutation_probability' in config_dic.keys():
            algorithm = GeneticAlgorithm(
                problem=problem,
                population_size=config_dic['population_size'],  # 10
                offspring_population_size=config_dic['offspring_population_size'], #10
                mutation=IntegerPolynomialMutation(probability=config_dic['mutation_probability'], distribution_index=config_dic['mutation_distribution_index']),
                crossover=IntegerSBXCrossover(probability=config_dic['crossover_probability'], distribution_index=config_dic['crossover_distribution_index']),  # 0.9, 20
                selection=BestSolutionSelection(),
                termination_criterion=StoppingByEvaluations(max_evaluations=config_dic['max_evaluations'])  # 500
            )
        else:
            algorithm = GeneticAlgorithm(
                problem=problem,
                population_size=config_dic['population_size'],  # 10
                offspring_population_size=config_dic['offspring_population_size'], #10
                mutation=IntegerPolynomialMutation(probability=1.0/problem.number_of_variables, distribution_index=config_dic['mutation_distribution_index']),
                crossover=IntegerSBXCrossover(probability=config_dic['crossover_probability'], distribution_index=config_dic['crossover_distribution_index']),  # 0.9, 20
                selection=BestSolutionSelection(),
                termination_criterion=StoppingByEvaluations(max_evaluations=config_dic['max_evaluations'])  # 500
            )

        algorithm.run()

        result = algorithm.get_result()
        result_int = result.variables
        solution = []
        for l in range(len(result_int)):
            solution.append(dec2bin(result_int[l],len(config_dic['inputID'])))

        #get count_times
        valid_input = config_dic['valid_input']
        pt = config_dic['p']
        count = []
        for l in range(len(solution)):
            count_times = 0
            for i in range(len(valid_input)):
                if valid_input[i] == solution[l] and pt[l] > 0:
                    count_times += 1
            count.append(count_times*100)

        # GenerateUnitTest.generateUnitTestClass(config_dic['module_name'], config_dic['inputID'], config_dic['outputID'], config_dic['num_qubit'], result_int, solution, count, config_dic['program_folder'],ROOT)


        solution.insert(0, 'Solution')
        obj = [-result.objectives[0][0]]

        obj.insert(0,'Fitness')
        total_time = [algorithm.total_computing_time]
        total_time.insert(0, 'Total time')

        for l in range(len(solution)):
            solution_sheet.cell(row=1,column=l+1,value=solution[l])
        for l in range(len(obj)):
            solution_sheet.cell(row=2,column=l+1,value=obj[l])
        for l in range(len(total_time)):
            solution_sheet.cell(row=3,column=l+1,value=total_time[l])

        wb.save(config_dic['program_folder']+ ROOT +config_dic['module_name']+'_result.xlsx')

        print("\n")
        print('Computing time: ', algorithm.total_computing_time)
        print('File '+config_dic['module_name']+'_result.xlsx'+' is generated.')

